function [id_rejected,BF,esttau2,Lhat,estpizero,estsigma]=Make_Analysis(datamatrix,times,repliche,L,vi,...
    lambda,type_fun,type_method,type_sigma,type_pizero,type_estdegree,type_test,type_mod,est_par_ig,...
    gammino,soglia,x0,x1,estsigmaC,estpizeroC)
%                                                                                                                                                                                                             
%    [id_rejected,BF,esttau,Lhat,estpizero,estsigma]=Make_Analysis(datamatrix,times,repliche,L,vi,lambda,
%    type_fun,type_method,type_sigma,type_pizero,type_estdegree,type_test,type_mod,est_par_ig,gammino,soglia,x0,x1); 
 
%    This routine performs the Bayesian Analysis proposed in the article "Bayesian approach to estimation and 
%    testing in time course microarray experiments" by C. Angelini, D. De Canditiis and M. Pensky.   
%    The program will identify differentially expressed genes in the  time-course microarray data datamatrix,
%    will rank the selected genes  and will estimate them in a given orthogonal basis function.
%    
%
%    Input:  
%      datamatrix     :2D data array; each row representing a gene expression profile over the time grid and each
%                      column a microarray
%      times          :1D array of the  time grid points
%      repliche       :1D array of the same size of times where each element is the number of replication at that time point
%      L              :maximum expected degree of polynomial representing the gene expression profile  
%      vi             :expected vi-time differentiability for the gene expression profile
%      lambda         :Poisson parameter for the prior probability on the polynomial degree
%      type_fun       :basis function
%                      'legendre'
%                      'fourier'              
%      type_method    :Type of method
%                       1 --> delta distribution on sigma^2;
%                       2 --> IG distribution on sigma^2;
%                       3 --> Exponential dist on sigma^2
%                   
%      type_sigma     :option on how to estimate sigma.
%                       'standard'--> standard deviation of the noise
%                       'mad' --> median absolute deviation of the noise.
%      type_pizero    :option on how to estimate the pizero. 
%                       'universal' --> (with an Hard Universal Threshold)
%                       'sure' --> (Adaptive Threshold Selection Using Principle of SURE)
%                       'EBL'; --> ( Empirical Bayes threshold)
%      type_estdegree :option on how to estimate the degree. 
%                       'mean' --> (posterior mean estimation )
%                       'map' --> (Maximum posterior estimation)
%      type_test      :option on which test procedure is used.
%                       'binomial'
%                       'standard_odds'
%                       'standard_BF'
%                       'Pois2';
%                       'Full';
%      type_mod       :option on multiple testing hypothesis
%                      'global' 
%                      'Step-Up';
%                      'Step-Down';
%      est_par_ig     :only for type_method==2  option on how to estimate parameter of IG: 1, 2 ,3
%                      '1' --> gammino is given and b is evaluated s.t. the prior mean of sigma is the estimated sigma
%                      '2' --> gammino is obtained as the parameter estimate for gamma distributed data applied over
%                      the inverse of the sampled variances of the microarrays
%                      '3' --> gammino and b are estimated simultaneously by  MLE using all the data under a 'soglia'
%      gammino        :is given only if est_par_ig==1
%      soglia         :is given only if est_par_ig==3
%      x0,x1          : are the interval where the minimization of the marginal pdf is done to estimate tau2
 
 
%
%    Output: 
%       id_rejected   :1D vector containing the indexes of the rejected hypothesis, i.e. of the differentially expressed genes
%       BF            :1D vector containing the corresponding Bayes Factor
%       esttau2        :1D vector containing the corresponding estimated tau^2
%       Lhat          :1D vector containing the corresponding estimated degree of polynomials representig the genes in the selected bass
%       estpizero     :a scalar representing the estimated proportion of differentially expressed gene
%       estsigma      :a scalar representing the estimated noise variance
%
 
% Reference: C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

% if nargin<17
% x0=10^(-8);    
% x1=100;
% end

Trange=[0,max(times)];
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % SEPARATION OF THE MATRIX IN TWO SUBMATRICES: with and without missing data
% 
 
id_full=find(sum(isnan(datamatrix),2)==0);
id_miss=find(sum(isnan(datamatrix),2)~=0);

subdata_full=datamatrix(id_full,:);
subdata_miss=datamatrix(id_miss,:);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % ESTIMATION GLOBAL PARAMETERS sigma and pi_zero
% 
fprintf('Estimating sigma2 and pi_0......\n') 





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(type_sigma)
    [estsigma,ss]=sigmaest(subdata_full,type_sigma);
else
    estsigma=estsigmaC;
end
if ~isempty(type_pizero)
    estpizero=pizeroest(subdata_full,estsigma,type_pizero);
else
    estpizero=estpizeroC;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % construction of matrices PHI, Q and q for the Full data matrix
% 
[Q]=make_matrix_Q(L,vi,type_fun); 
 
[PHI]=make_matrix_PHI(L,times,repliche,type_fun);

[q]=make_vector_q(subdata_full,L,times,repliche,type_fun);
% 
Zzero=sum(subdata_full.^2,2);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % ESTIMATION OF SPECIFIC PARAMETERS on data without missings
% 
if type_method==2
 if est_par_ig==1
   b=(gammino-1)*estsigma^2;
 elseif est_par_ig==2
   [a]=gamfit(1./ss.^2);
   gammino=a(1);
   b=1/a(2);
 elseif est_par_ig==3
  iii=find(Zzero<soglia); 
  [a]=fminsearch(@(x) mymargpdf_small_zzero(x,sum(repliche),Zzero,iii),[1,1]);
  gammino=a(1);
  b=a(2);
 end
end

if type_method==3
 mu=(sum(repliche)-1)/estsigma^2;
end

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % ESTIMATION OF VECTOR tau_i^2
% 

fprintf('Estimating tau2 on data without missings......\n')
if type_method==1
    param=[];
elseif type_method==2
    param=[b,gammino,sum(repliche)];
elseif type_method==3
    param=mu;
end

esttau2_full=estimationtau_fast(q,PHI,Q,Zzero,estsigma,vi,lambda,type_method,param,x0,x1); %%%
%
% degree estimate L_i  and evaluation of BF
%
fprintf('Estimating the polinomial degree and computing the BF on data without missings......\n')
[Lhat_full,BF_full]=degreestimate(q,PHI,Q,Zzero,vi,estsigma,estpizero,esttau2_full,lambda,type_method,param,type_estdegree);

clear q
clear PHI
clear Zzero

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %  ESTIMATION OF SPECIFIC PARAMETERS on data with missings
% 
if ~isempty(id_miss)
    fprintf('Estimating tau2 on data with missings......\n')

    esttau2_miss=zeros(length(id_miss),1);
    Lhat_miss=zeros(length(id_miss),1);
    BF_miss=zeros(length(id_miss),1);
    fprintf('Estimating the polinomial degree and computing the BF on data with missings......\n')

 for i=1:length(id_miss)
    repliche_miss=repliche;
    times_miss=times;
    ipos=find(isnan(subdata_miss(i,:)));

    for j=1:length(ipos)
     irem=[];
     ix=find(cumsum(repliche)>=ipos(j));
     irem=ix(1);
     clear ix
     repliche_miss(irem)=repliche_miss(irem)-1;
    end
    izero=find(repliche_miss==0);
    if ~isempty(izero)
    times_miss(izero)=[];
    repliche_miss(izero)=[];
    end

[PHI_miss]=make_matrix_PHI(L,times_miss,repliche_miss,type_fun,Trange);
subdata_miss_dum=subdata_miss(i,:);
subdata_miss_dum(ipos)=[];
[q_miss]=make_vector_q(subdata_miss_dum,L,times_miss,repliche_miss,type_fun,Trange);
Zzero_miss=diag(subdata_miss_dum*subdata_miss_dum');

if type_method==1
    param=[];
elseif type_method==2
    param=[b,gammino,sum(repliche_miss)];
elseif type_method==3
    param=mu;
end

esttau2_dum=estimationtau_fast(q_miss,PHI_miss,Q,Zzero_miss,estsigma,vi,lambda,type_method,param,x0,x1); %%%
esttau2_miss(i)=esttau2_dum;

[Lhat_dum,BF_dum]=degreestimate(q_miss,PHI_miss,Q,Zzero_miss,vi,estsigma,estpizero,esttau2_miss(i),lambda,type_method,param,type_estdegree);

 Lhat_miss(i)=Lhat_dum;
 BF_miss(i)=BF_dum;
 clear param
 clear BF_dum Lhat_dum   
 clear repliche_miss times_miss ipos subdata_miss_dum
 clear PHI_miss q_miss Zzero_miss  
 end % close over i
end % close if over id_miss

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Testing the null hypothesis that the function is zero against the alternative that it is not zero
%%%%%%%

esttau2(id_full)=esttau2_full;
Lhat(id_full)=Lhat_full;
BF(id_full)=BF_full;

if ~isempty(id_miss)
esttau2(id_miss)=esttau2_miss;
Lhat(id_miss)=Lhat_miss;
BF(id_miss)=BF_miss;
end

    [id_rejected]=make_test(BF,estpizero,type_test,type_mod);
if strcmp(type_test,'Full')
    fprintf('Not Testing Hypotheses......\n just giving the full BF List '); 
else
    fprintf('Testing Hypotheses......\n');

    fprintf('Selection Done......\n'); 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%