function [coeffs,shat]=Make_Estimation(datamatrix,index,BF,Lhat,estpizero,esttau2,L,vi,type_fun,times,repliche)

%   [coeffs,shat]=Make_Estimation(datamatrix,index,BF,Lhat,estpizero,esttau2,L,vi,type_fun,times,repliche)  
 
%    This routine evaluates for each gene declared significant whose indexes are in vector "index"
%    the coefficients representing its profile in the selected basis and the time value of its polynomials
%    approximation over the grid time knots (of course with no replication)
%
%    Input:  
%      datamatrix     :2D data array; each row representing a gene expression profile over the time grid and each
%                      column a microarray
%      index          :1D array of the indexes of the gene profiles we want to estimate (they are the output of the
%                      routine Make_Analysis ,i.e. the differentially expressed genes
%       BF            :1D vector (same size of index) with the corresponding Bayes Factor 
%       Lhat          :1D vector (same size of index) with the corresponding estimated degree of polynomials 
%                      representing the genes in the selected bass
%       estpizero     :a scalar representing the estimated proportion of differentially expressed gene
%       esttau2        :1D vector (same size of index) with the corresponding estimated tau^2
%      L              :maximum expected degree of polynomial representing the gene expression profile  
%      vi             :expected vi-time differentiability for the gene expression profile
%      type_fun       :basis function choice
%                      'legendre'
%                      'fourier'              
%      times          :1D array of the  time grid points
%      repliche       :1D array of the same size of times where each element is the number of replication at that time point
%
%    Output: 
%       coeffs    :2D array where each row has the estimated coefficients of the corresponding gene profile
%       shat      :2D array where each row has the estimated value profile in the time instants times
%
 
% Reference: C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"


fprintf('Estimating selected profiles.....\n')

Trange=[0,max(times)];

design_matrix=make_design_matrix(L,times,type_fun);

id_full=find(sum(isnan(datamatrix(index,:)),2)==0);
id_miss=find(sum(isnan(datamatrix(index,:)),2)~=0);

subdata_full=datamatrix(index(id_full),:);
subdata_miss=datamatrix(index(id_miss),:);

[Q]=make_matrix_Q(L,vi,type_fun); 
%
% %construction of PHI matrix for the Full data
% 
[PHI]=make_matrix_PHI(L,times,repliche,type_fun);
%
% % construction of matrix q for the Full data
% 
[q]=make_vector_q(subdata_full,L,times,repliche,type_fun);

for i=1:length(id_full) 
    estcoefficient=[];
    estcoefficient=estcoeff(i,PHI,Q,q,BF(id_full(i)),esttau2(id_full(i)),Lhat(id_full(i)),estpizero);
    shat(id_full(i),:)=estcoefficient*design_matrix(1:Lhat(id_full(i))+1,:);
    coeffs(id_full(i),1)={estcoefficient};
end

if ~isempty(id_miss)
 for i=1:length(id_miss)
    repliche_miss=repliche;
    times_miss=times;
    ipos=find(isnan(subdata_miss(i,:)));
    for j=1:length(ipos)
     irem=[];
     ix=find(cumsum(repliche)>=ipos(j));
     irem=ix(1);
     clear ix
     repliche_miss(irem)=repliche_miss(irem)-1;
    end
    izero=find(repliche_miss==0);
   
    if ~isempty(izero)
    times_miss(izero)=[];
    repliche_miss(izero)=[];
    end
 
    % %construction of PHI matrix for miss data
    % 
    [PHI_miss]=make_matrix_PHI(L,times_miss,repliche_miss,type_fun,Trange);
%
% % construction of matrix q for the Miss data
% 
subdata_miss_dum=subdata_miss(i,:);
subdata_miss_dum(ipos)=[];

   [q_miss]=make_vector_q(subdata_miss_dum,L,times_miss,repliche_miss,type_fun,Trange);
   estcoefficient=[];
    estcoefficient=estcoeff(1,PHI_miss,Q,q_miss,BF(id_miss(i)),esttau2(id_miss(i)),Lhat(id_miss(i)),estpizero);
    shat(id_miss(i),:)=estcoefficient*design_matrix(1:Lhat(id_miss(i))+1,:);
    coeffs(id_miss(i),1)={estcoefficient};
 end %end over for i=1:lenght(id_miss)
end

fprintf('Estimation Done.....\n')

 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
