function [Lhat,BF]=degreestimate(q,PHI,Q,zzero,vi,sigma,pizero,tau2,lambda,caseoption,param,type_estdegree)
%
%  [Lhat,BF]=degreestimate(q,PHI,Q,zzero,vi,sigma,pizero,tau2,lambda,caseop
%  tion,param,type_estdegree)
%
%  This routine estimates the degree of the polynomial representing each gene profile in the selected basis function 
%  moreover this routine gives the Bayes Factor of each gene profile
%
%  Input:
%      q             :1D array output of subroutine make_vector_q
%      PHI           :2D array output of subroutine make_matrix_PHI
%      Q             :2D array output of subroutine make_matrix_Q
%      Zzero         :1D vector (of the same size of q) which contain the l_2-norm of each row of the datamatrix
%      sigma         :standard deviation of the noise
%      vi            :expected degree of differentiability of the function profile
%      sigma         :standard deviation of the noise
%      pizero        :proportion of not differentially expressed genes
%      tau2          :1D vector of the tau^2_i
%      lambda        :Poisson parameter for the prior probability on the polynomial degree
%      caseoption    :Type of method
%                      1 --> delta distribution on sigma^2;
%                      2 --> IG distribution on sigma^2;
%                      3 --> Exponential dist on sigma^2
%      param          :specific prior parameter
%                      if caseoption==1  param=[]
%                      if  caseoption==2  param=[b, gammino]
%                      if  caseoption==3  param=[mu]
%      type_estdegree :option on how to estimate the degree. 
%                      'mean' --> (posterior mean estimation )
%                      'map' --> (Maximum posterior estimation)
 
%    Output: 
%      Lhat      :1D array of estimated degree of dimension  number_genes x 1
%      BF        :1D array of Bayes factor of dimension number_gene x 1
%  
 
% Reference: C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

if nargin<12
   type_estdegree='map';
end

[n,d]=size(q);
L=d-1;

poiss=exp(-lambda).*(lambda.^(0:L))./factorial(0:L);% probability for Poisson.
poiss=poiss./sum(poiss); % renormalization for accouting the truncation
fact=factorial(1:L+1);

app1=zeros(1,L+1);
app2=zeros(1,L+1);

for i=1:n
  H1=PHI+Q/tau2(i);
  H2=tau2(i)*PHI+Q;
  
  for k=0:L
   Hinv=inv(H1(1:k+1,1:k+1));
   Hdet=sqrt(det(H2(1:k+1,1:k+1)));
   dummy=q(i,1:k+1)*Hinv*q(i,1:k+1)';
   num2=fact(k+1)^vi/Hdet;
   
   if caseoption==1
      num1=exp(dummy/(2*sigma^2));
      app1(i,k+1)=num1*num2;
      app2(i,k+1)=1;
   elseif caseoption==2
       b=param(1);
       gammino=param(2);
       m=param(3);
      num1=(1+(zzero(i)-dummy)/(2*b))^(-(m/2+gammino));
      app1(i,k+1)=num1*num2;
      app2(i,k+1)=(1+zzero(i)/(2*b))^(-(m/2+gammino));
   elseif caseoption==3
      mu=param(1);
      num1=exp(-sqrt(mu*(zzero(i)-dummy)));
      app1(i,k+1)=num1*num2;
      app2(i,k+1)=exp(-sqrt(mu*zzero(i)));
   end
    Hinv=[];
  end
end

%
%construction of the  matrix with the numerator of p(l| z_jk^(i))
%l=0,1,2,..L in each line
for k=0:L
    probdegree(:,k+1)=(app1(:,k+1)*(1-pizero)+app2(:,k+1)*pizero).*poiss(k+1); %%%not normalized
end

 Lhat=zeros(n,1);

 if strcmp(type_estdegree,'map') %%%%%%%estimation of degree by posterior mode %%%%%%%%%%%%%%%    
 maximum=max(probdegree,[],2);
  for i=1:n
   % index=[];
   % lengthind=[];
    index=find(probdegree(i,:)==maximum(i));
    lengthind=length(index);
    Lhat(i)=index(lengthind)-1;
  end
clear index
elseif strcmp(type_estdegree,'mean') %%%%%%%estimation of degree by posterior mean %%%%%%%%%%%%%%%
    LL=zeros(n,1);
  for i=1:n
   probdegree(i,:)=probdegree(i,:)./sum(probdegree(i,:)); %for normalization purposes
   LL(i)=sum((0:L).*probdegree(i,:));
   Lhat(i)=round(LL(i)); %%%we round to the nearest integer to get a proper (integer) degree
  end
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% construction of the vector of BF
index=Lhat+1;

BF=zeros(n,1);
for i=1:n
  BF(i)=app2(i,index(i))/(app1(i,index(i))+eps); 
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
