function [D]=make_design_matrix(maxdegree,times,type_fun)

%
%  [D]=make_design_matrix(maxdegree,times,type_fun)
 
%  This routine create the design matrix D for a given maximum degree and over the grid of distinct time-points
%  without taking into account how many replications are in each time point. 
%  D has dimension M x (maxdegree+1), where M is the number of time instants.
%  
%
%  Input:  
%      maxdegree :a scalar representing the maximum degree of the  polynomial
%      times     :1D vector with the time knots
%      type_fun  :basis function
%                 'legendre'
%                 'fourier' 
%    Output: 
%      D         :design matrix of dimension (length(times)) x (maxdegree+1)
%     
% Reference: C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

if nargin<3
    type_fun='legendre';
end

if strcmp(type_fun,'legendre')
xtimes=2*(times)/max(times)-1; %%% Rescaling  [0,max(T)] to [-1 1] !!! 

ntimes=length(times);
p=zeros(maxdegree,ntimes);

for degree=0:maxdegree
 dummy=legendre(degree,xtimes)*sqrt((2*degree+1)/2); %%%In order to normalize in L2. The decay we have is for orthonormal polynomial
 p(degree+1,:)=dummy(1,:);
end

D=p;
elseif strcmp(type_fun,'fourier') 
    ftimes=[-fliplr(times) times];
    ntimes=length(ftimes);
    p=zeros(maxdegree,ntimes);
     
    xtimes=pi*(ftimes)/max(ftimes) ; 

    for degree=0:maxdegree
     if degree==0
      dummy=ones(size(xtimes))./sqrt(2*pi);  
     else
      dummy=cos(degree*xtimes)./sqrt(pi); 
     end
     p(degree+1,:)=dummy;
    end
   D=p;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
