function [Id_rejected]=make_test(BF,par,type,mod)
%
%  [Id_rejected]=make_test(BF,par,type,mod)
%
%  This routine apply the Bayesian multiple hypothesis testing procedure proposed in
%  Abramovich & Angelini 2006
%
%  Input:
%      BF          :1D vector of Bayes factor of dimension number_of_genes x 1
%      par         :par=pizero the proportion of not differentially expressed genes
%      type_test   :option on which test procedure is used.
%                   'binomial'
%                   'standard_odds'
%                   'standard_BF'
%                   'Pois2';
%                   'Full';
%      type_mod    :option on multiple testing hypothesis
%                  'global' 
%                  'Step-Up';
%                  'Step-Down';

%    Output: 
%      Id_rejected :1D array of the indexes of the selected genes (differentiall yexpressed)
%     
% Reference: 
%1)  C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"
%2)  F. Abramovich and C. Angelini (2006). "Bayesian maximum a posteriori
% multiple testing procedure" To appear Sankhya
 
if nargin<4
    mod=[];
end

if length(par)<2
    par=[par,1];
end
 
n=length(BF); %%%number of hypotheses to be tested

if strcmp(type,'standard_odds')
   pizero=par(1);
   cut_off=par(2);
   PostOdds=BF*(pizero./(1-pizero));
   Id_rejected=find(PostOdds<cut_off);  %%% No multiplicity control, the test is made on the posterior odds
elseif strcmp(type,'Full')
   Id_rejected=[1:n];%%% No multiplicity control, no the test is made!
elseif strcmp(type,'standard_BF')
   cut_off=par(2);
   Id_rejected=find(BF<cut_off);  %%% No multiplicity control, the test is made on the posterior odds
elseif strcmp(type,'binomial')
   pizero=par(1);
   alpha=1-pizero;
   Id_rejected=find(BF<(alpha/(1-alpha)));  %%% Multiplicity control with Binomial Prior, the test is made on the BF; the parameter of the Binomial is alpha=1-pizero.
                                             %%% note that When pi(k) the prior on the number of false hypothesis is Binomial then the Bayesian global MAP and the step-wise  
elseif strcmp(type,'Pois2')
   pizero=par(1);
   lambda2=n*pizero;
   thresh=([1:1:n]./lambda2);
   if strcmp(mod,'Step-Up') | strcmp(mod,'Step-Down')
     [R,Id_rejected]=make_step_MAP_test(BF,thresh,mod);
   elseif strcmp(mod,'global') 
     pizero=par(1);
     lambda2=n*pizero;
     [BF_sorted, index] = sort(BF);
     index_up=[1:n];
     index_down=[n:-1:1];
     log_index_up=log(index_up);
     log_index_down=log(index_down);

     F1=cumsum(log_index_up)-sum(log_index_up);
     F2=(index_down-1)*log(lambda2);
     F3=cumsum(log(BF_sorted));
     pi0=n*log(lambda2)-sum(log_index_up);
     F=F2+F1-F3;
     F=[pi0 F];
     R=find(F==max(F)); %%%%See Global algorithm in Abramovich & Angelini 2005
     R=R-1;
     Id_rejected=index(1:R);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2006. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2006.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
