function [q]=make_vector_q(datamatrix,maxdegree,times,repliche,type_fun,Trange)

%
%  [q]=make_vector_q(datamatrix,maxdegree,times,repliche,type_fun,Trange)

%
%  This routine computes the vector q for a given maximum degree and over the complete time grid.
%  Note that q has dimension number_genes x 1 where number_genes is the number of raw in datamatrix
%
%  Input:
%      datamatrix :2D data array; each row representing a gene expression profile over the time grid and each
%                      column a microarray
%      maxdegree  :a scalar representing the maximum degree of the  polynomial
%      times      :1D vector with the time knots
%      repliche   :1D vector (of the same size of times) with the replicates for each time instant
%      type_fun   :basis function
%                 'legendre'
%                 'fourier' 
%      Trange     :extremes of the Time interval
%    Output: 
%      q          :1D array of dimension  number_genes x 1 where number_genes is the number of raw in datamatrix
%     
% Reference: 
% C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

if nargin<6
    Trange(1)=0;
    Trange(2)=max(times);
end

ntobs=sum(repliche);
p=zeros(maxdegree,ntobs);

if nargin<5
    type_fun='legendre';
end

if strcmp(type_fun,'legendre')
xtimes=2*(times)/(Trange(2)-Trange(1))-1; %%% Rescaling [T(1),T(2)] to [-1 1] !!! 

T=length(times); 
mtimes=[];%
for k=1:T
    for l=1:repliche(k)
        mtimes=[mtimes xtimes(k)];
    end
end

for degree=0:maxdegree
 dummy=legendre(degree,mtimes)*sqrt((2*degree+1)/2); %%%In order to normalize in L2. The decay we have is for orthonotmal polynomial;
 p(degree+1,:)=dummy(1,:);
end

q=datamatrix * p';

elseif strcmp(type_fun,'fourier')
    ftimes=[-fliplr(times) times];
    repliche=[fliplr(repliche) repliche];
    ntobs=sum(repliche);
    p=zeros(maxdegree,ntobs);
     
    xtimes=pi*(ftimes)/Trange(2);  

    T=length(xtimes); 
    mtimes=[];%
   for k=1:T
    for l=1:repliche(k)
        mtimes=[mtimes xtimes(k)];
    end
   end
    
    for degree=0:maxdegree
     if degree==0
      dummy=ones(size(mtimes))./sqrt(2*pi);  
     else
      dummy=cos(degree*mtimes)./sqrt(pi); 
     end
     p(degree+1,:)=dummy;
    end
    datamatrix=[fliplr(datamatrix) datamatrix];
    q=datamatrix*p';
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

