function fun=mymarginalpdf(x,PHI,Q,q,zzero,lambda,sigma,vi,caseoption,param,fact)

%
%  fun=mymarginalpdf(x,PHI,Q,q,zzero,lambda,sigma,vi,caseoption,param,fact)
%
%  This routine evaluates a function proportional to the  marginal pdf of the data for each gene.
%  This marginal pdf will be maximized in order to estimate tau^2
%
%  Input:
%      x          :x=tau_i^2 
%      q          :a scalar = q(i) where q is the output of subroutine make_vector_q
%      PHI        :2D array output of subroutine make_matrix_PHI
%      Q          :2D array output of subroutine make_matrix_Q
%      Zzero      :a scalar = Zzero(i) representing the l_2-norm of the i-th row of the datamatrix
%      sigma      :standard deviation of the noise
%      vi         :expected degree of differentiability of the function profile
%      caseoption :Type of method
%                   1 --> delta distribution on sigma^2;
%                   2 --> IG distribution on sigma^2;
%                   3 --> Exponential dist on sigma^2
%      param      :specific prior parameter
%                   if caseoption==1  param=[]
%                   if  caseoption==2  param=[b, gammino]
%                   if  caseoption==3  param=[mu]
%      fact       :factorial product
%    Output: 
%     fun         :a scalar p(z_i)
%     

% Reference: 
% C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

     degree=size(PHI,1)-1;
     H1=PHI+Q/x;
     H2=(x)*PHI+Q;
     app=zeros(1,degree+1);
   
  if nargin<11
     fact=factorial(0:degree+1); 
  end
     
     for k=0:degree             
        Hinv=inv(H1(1:k+1,1:k+1));
        Hdet=sqrt(det(H2(1:k+1,1:k+1)));
        dumm=(fact(k+2)^vi/Hdet)*(lambda^(k)/fact(k+1));         
        if caseoption==1
           app(k+1)=exp((q(1:k+1)*Hinv*q(1:k+1)')/(2*sigma^2) )*dumm; %
        elseif caseoption==2
          b=param(1);
          gammino=param(2);
          m=param(3);
           app(k+1)=(1+(zzero-q(1:k+1)*Hinv*q(1:k+1)')/(2*b))^(-(m/2+gammino))*dumm; % 
        elseif caseoption==3
            mu=param(1);
            app(k+1)=exp(- sqrt(mu*(zzero-q(1:k+1)*Hinv*q(1:k+1)')))*dumm; % 
        end
        Hinv=[];
        Hdet=[];
      %      clear Hinv; clear Hdet;
     end

 fun=-sum(app);

 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
