function  estpizero=pizeroest(datamatrix,sigma,type)
%
%  estpizero=pizeroest(datamatrix,sigma,type)
%
%  This routine estimates parameter pizero which represent the proportion of not differentially expressed genes
%
%  Input:  
%      datamatrix :2D data array each column representing a microarray
%      sigma      :standard deviation of the noise 
%      type       :option on how to estimate the pizero
%                 'universal' --> (with an Hard Universal Threshold)
%                 'sure' --> (Adaptive Threshold Selection Using Principle of SURE)
%                 'EBL'; --> ( Empirical Bayes threshold)
%    Output: 
%      estpizero  :estimated value of pizero
%     
 
% Reference: 
% C. Angelini, D. De Canditiis, M. Mutarelli and M. Pensky (2007)."Bayesian approach to estimation and 
% testing in time course microarray experiments"

if nargin<3
    type='EBL';
end

[n,m]=size(datamatrix);

if strcmp(type,'universal')  
  for iexp=1:m
   deltathres=sigma*sqrt(2*log(n));  
   S=HardThresh(datamatrix(:,iexp),deltathres);
   estpizero_dum(iexp)=1-(nnz(S)/n);
   clear S
  end
elseif strcmp(type,'sure')
   for iexp=1:m
    thresh(iexp)=ValSUREThresh(datamatrix(:,iexp)'/sigma); 
    deltathres=thresh(iexp)*sigma;
    S=HardThresh(datamatrix(:,iexp),deltathres);
    estpizero_dum(iexp)=1-(nnz(S)/n);
    clear S
   end
elseif strcmp(type,'EBL') % Empirical Bayes threshold see Ebayes thresh by J&S 2005
 for iexp=1:m
   [w]= wfromx(datamatrix(:,iexp)/sigma); %%laplace with parameter a=0.5
   estpizero_dum(iexp)=1-w;
 end 
elseif strcmp(type,'em')
  for iexp=1:m
   par=em_s([0.1,6],sigma,datamatrix(:,iexp)); 
   w=par(1);
   estpizero_dum(iexp)=1-w;
  end
end
% 
 estpizero=mean(estpizero_dum);
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by C.Angelini, L. Cutillo, D.De Canditiis, M. Mutarelli and M. Pensky, 2007. 
% Comments? e-mail c.angelini@iac.cnr.it, l.cutillo@na.iac.cnr.it, d.decanditiis@iac.cnr.it
% Copyright:  I.A.C. "M. Picone" CNR (ITALY), 2007.  
%
% Permission to use, copy, modify, and distribute the 
% software for any purpose without fee is hereby granted, provided 
% that this entire notice is included in all copies of any software 
% which is or includes a copy or modification of this software and in 
% all copies of the supporting documentation for such software.
% 
% This software is being provided "as is", without any express or 
% implied warranty.  In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

